/********************************************************************************
*										*
*            (c) gion electronic Hardware & Systementwicklung			*
*		    D-86199 Augsburg Hessingstr.2				*
*			www.gion-electronic.de					*
*										*										*
*********************************************************************************
*										*
*    FILENAME      : PWM13_6.C  	       				        *
*										*
*    PROJECT       : PWM Funktion fr Motorsteuerung ect. Kennlinie teachbar    *
*		                                                                *
*		                                                        	*
*		                                				*
*    KUNDE	   :  DG8CAJ                    				*
*										*
*    BESCHREIBUNG  :  nachfolgend                             	        	*
*		     								*
*------------------------------------------------------------------------------	*
*										*
*    VERSION       : 6.0					   		*
*		          							*
*			  							*
*			  							*
*										*
*    BEARBEITER    : R.GION, gion electronic Hessingstr.2 86199 Augsburg	*
*		     Tel.: 0821-9988029 Fax: 0821-9988030 Mob.: 0178-9143677	*
*										*
*    DATUM         : 17.01.2016							*
*    MOD.          : 21.01.2016        						*
*										*
********************************************************************************/

//	Chip type           : ATTiny85
//	Program type        : Application
//	Clock frequency     : 8 MHz internal oscillator
//	Memory model        : Small
//	External SRAM size  : 0
//	Data Stack size     : 256
//      Compiler            : CodeVision AVR Ver 2.05.0
//      Code memory used    : 29%
//      Programmer type     : AVR ISP
//
//*******************************************************************************

/**************************** Funktionsbeschreibung *****************************

konfigurierbarer PWM - Controller:

mittels Pin F1 (PB3) und Pin F2 (PB4) kann die PWM Frequenz eingestellt werden:

        F1      F2      Frequenz
        --------------------------
        low  | low  |  138 Hz
        high | low  |  550 Hz
        low  | high | 4,4 kHz
        high | high | 35 kHz
        
mittes Pin Teach (PB1) kann der Analog Eingangsbereich (ADC0) eingestellt werden:

In dieser version muss der erste teach Wert kleiner sein als der zweite teach Wert!!!

z.B.:   1V entspricht 0% PWM           default ist 0V-0% und 5V-100%
        4V entspricht 100% PWM
        
Einstellung wie folgt:

- Versorgungsspannung anlegen (hier 5V)                
- 1V an den Analogeingang anlegen
- Pin Teach fr ca. 1s nach low legen
- 4V an den Analogeingang anlegen
- Pin Teach fr ca. 1s nach low legen
- fertig
- die Skalierung wird intern dauerhaft gespeichert.

- der erste Wert kann auch grsser sein als der zweite Wert, die bertragungskennlinie
  erscheint dann invertiert

Wenn die beiden Eingangswerte (1V/4V) weniger als 10% auseinanderliegen
so wird die Grundeinstellung 0V-0% und 5V-100% eingestellt (default).

Liegt bei Anlegen der Versorgungsspannung der Pin Teach an Masse so wird
die Grundeinstellung 0V-0% und 5V-100% eingestellt (default).

Pinbelegung

F1 -            Pin 2
F2 -            Pin 3
Teach -         Pin 6
PWM Ausgang -   Pin 5
Analog in -     Pin 7
Vcc -           Pin 8
GND -           Pin 4                                                         */

//*******************************************************************************

#include <stdio.h>
#include <delay.h>
#include <tiny85.h>
#include <stdlib.h>

#define f_select_1      PINB.3	// frequency select input 1
#define f_select_2      PINB.4	// frequency select input 2
#define pwm_out         PORTB.0
#define teach           PINB.1	// teach key

#define uchar	unsigned char
#define uint	unsigned int
#define ulong   unsigned long

float pwm_value;                    // 
float adc_value;
float low_scale;
float high_scale;
float pwm_scale = 1;

eeprom float ee_low_scale = 0;
eeprom float ee_high_scale = 1023;
eeprom float ee_pwm_scale = 1;


bit low_done = 0;
bit scale_done = 0;
bit reverse = 0;

void sys_init(void);		        // Systemvariablen initialisieren
void get_adc(void);                     // read the ADC

/********************************************************************************/
void sys_init()
{
  DDRB |= 0x01;                            // pwm output on / all other input
  PORTB |= 0x1A;                           // pullup on
  ADCSRA = 0xC4;		           // ADC enable - 125kHz
  TCCR0A = 0x85;                           // set timer 0 for fast pwm normal MODE 5
    
  if ( !f_select_1 & !f_select_2 )        // PWM frequency 61Hz
          TCCR0B = 0x04;                  
  else
  if ( f_select_1 & !f_select_2 )         // PWM frequency 244Hz
          TCCR0B = 0x03;                 
  else
  if ( !f_select_1 & f_select_2 )         // PWM frequency 1,95kHz
          TCCR0B = 0x02;                 
  else
  if ( f_select_1 & f_select_2 )          // PWM frequency 15,6kHz
          TCCR0B = 0x01;                 
  
    
}
/*******************************************************************************/
void get_adc(void)
{
  ADMUX = 1;			        // select chanel ADC0
  ADCSRA |= 0x40;			// start conversation
  
  while ( ADCSRA & 0x40);		// wait for conversion finished
  adc_value = ADCW;
  
}
/********************************************************************************/
/*										*/
/*			H A U P T P R O G R AM M				*/
/*										*/
/********************************************************************************/

void main(void)
 
{
     
  sys_init();                           // init system, timer
  
  if ( !teach )                         // if teach pin held low while power on
        {                               // then load default
         ee_low_scale = 0;
         ee_high_scale = 1023;
         ee_pwm_scale = 1;
        } 
  while ( !teach );;
  
  low_scale = ee_low_scale;             // restore last stored values
  high_scale = ee_high_scale;
  pwm_scale = ee_pwm_scale;
      
  for(;;)                         	// loop forever !
  {                             		
      
      
      if ( !teach & !low_done & !scale_done)                     // teach request?
        {
         delay_ms(500);
         if ( !teach )
            {
             get_adc();
             low_scale = adc_value;
             ee_low_scale = low_scale;
             low_done = 1;
             while(!teach);;
             delay_ms(100);
            } 
         }
       
       if ( !teach & low_done & !scale_done)                     // teach request?
        {
         delay_ms(500);
         if ( !teach )
            {
             get_adc();
             high_scale = adc_value;
             if ( fabs(high_scale - low_scale) < 100 | fabs(low_scale - high_scale) < 100)
                 {
                  low_scale = 0;                // set back to default if span too small
                  high_scale = 1023;
                  pwm_scale = 1;
                  ee_low_scale = low_scale;
                  ee_high_scale = high_scale;
                  ee_pwm_scale = pwm_scale;
                  low_done = 0;
                  scale_done = 1;
                  while(!teach);;
                 } 
             
             else                             // calculate new scale
                {                
                 if ( high_scale > low_scale )
                     {
                      ee_high_scale = high_scale;
                      low_done = 0;
                      scale_done = 1;
                      pwm_scale = 1023 / ( high_scale - low_scale );// Scaling berechnen
                      ee_pwm_scale = pwm_scale;
                      reverse = 0;
                     while(!teach);;
                     }
                 
                 if ( high_scale < low_scale )
                     {
                      ee_high_scale = high_scale;
                      low_done = 0;
                      scale_done = 1;
                      pwm_scale = 1023 / ( low_scale - high_scale );// Scaling berechnen
                      ee_pwm_scale = pwm_scale;
                      reverse = 1;
                     while(!teach);;
                     }
                
                } 
            }
         }
      get_adc();                // read ADC 10bit
      
      if (!reverse)                              // if low_value < high value
            adc_value = adc_value - low_scale;
      else adc_value = adc_value - high_scale;  // if low_value > high_value
      
      
      if ( adc_value <= 0 )                     // keep value in range
                adc_value = 0;
      pwm_value = (adc_value * pwm_scale) / 4;
      if ( pwm_value >= 255 )                   // keep value in range
                pwm_value = 255;
      if (!reverse)
        OCR0A = (uchar)(pwm_value);            // refresh pwm output
      else OCR0A = ~((uchar)(pwm_value));     // refresh pwm output complement
      
      if ( pwm_value == 0 );                   // forced clear
                pwm_out = 0;
            
  } 
  
}

/********************************************************************************/

/* end of source file */